//
// Matrix Class
//

#include <iostream>
#include <fstream>
#include <cmath>
#include <cstdio>

using namespace std;

#include "Mat.h"

#define TMPBUFFLEN      80
static char tmpbuff[TMPBUFFLEN];

// 󥹥ȥ饯1
Mat::Mat(int row, int col)
{
   _row = row;
   _col = col;
   mat = new VecR[row];
   for (int i = 0; i < row; i++)
   {
      mat[i].renew(col);
      for (int j = 0; j < col; j++)
	 mat[i].vec[j] = 0.0;
   }
}

Mat::Mat(double a, double b, double c, double d, double e,
	 double f, double g, double h, double i)
{
   _row = _col = 3;
   mat = new VecR[_row];
   mat[0].vec[0] = a;
   mat[0].vec[1] = b;
   mat[0].vec[2] = c;
   mat[1].vec[0] = d;
   mat[1].vec[1] = e;
   mat[1].vec[2] = f;
   mat[2].vec[0] = g;
   mat[2].vec[1] = h;
   mat[2].vec[2] = i;
}

// 󥹥ȥ饯2
Mat::Mat(double *d, int row, int col)
{
   double	*ptr = d;
   _row = row;
   _col = col;
   mat = new VecR[row];
   for (int i = 0; i < row; i++)
   {
      mat[i].renew(col);
      for (int j = 0; j < col; j++)
	 mat[i].vec[j] = *ptr++;
   }
}

// 󥹥ȥ饯3
Mat::Mat(const VecC& c1, const VecC& c2, const VecC& c3)
{
   _row = c1.size();
   _col = 3;
   mat = new VecR[_row];
   for (int i = 0; i < _row; i++)
   {
      mat[i].vec[0] = c1[i];
      mat[i].vec[1] = c2[i];
      mat[i].vec[2] = c3[i];
   }
}

// ԡ󥹥ȥ饯
Mat::Mat(const Mat& M)
{
   _row = M._row;
   _col = M._col;
   mat = new VecR[_row];
   for (int i = 0; i < _row; i++)
   {
      mat[i].renew(_col);
      for (int j = 0; j < _col; j++)
	 mat[i].vec[j] = M[i].vec[j];
   }
}

// ǥȥ饯
Mat::~Mat()
{
   delete [] mat;
}

// ǡΥå
void
Mat::clear(double data)
{
   for (int i = 0; i < _row; i++)
      for (int j = 0; j < _col; j++)
	 mat[i].vec[j] = data;
}

// ɽؿ(ǽ)
void
Mat::print(ostream& s, const char *format)
{
   s << "\n";
   for (int i = 0; i < _row; i++)
   {
      for (int j = 0; j < _col; j++)
      {
         snprintf(tmpbuff, TMPBUFFLEN, format, mat[i][j]);
         s << tmpbuff << " ";
      }
      s << "\n";
   }
}

// ٥ȥ˽񤭹
void	Mat::setcvec(int n, VecC& V)
{
   assert((n >= 0) && (n < _col));

   for (int i = 0; i < _row; i++)
      mat[i].vec[n] = V[i];
}

// ޥȥꥯ黻(û)
Mat& Mat::operator += (const Mat& M)
{
   assert(M._row == _row && M._col == _col);
   for (int i = 0; i < _row; i++)
      for (int j = 0; j < _col; j++)
	 mat[i].vec[j] += M[i].vec[j];

   return *this;
}

// ޥȥꥯ黻()
Mat& Mat::operator -= (const Mat& M)
{
   assert(M._row == _row && M._col == _col);
   for (int i = 0; i < _row; i++)
      for (int j = 0; j < _col; j++)
	 mat[i].vec[j] -= M[i].vec[j];

   return *this;
}

// ޥȥꥯ黻(軻)
Mat& Mat::operator *= (const Mat& M)
{
   int		i, j;
   Mat		T(_row, _col);

   assert(M._row == _row && M._col == _col);

   for (i = 0; i < _row; i++)
      for (j = 0; j < _col; j++)
      {
	 T[i].vec[j] = 0.0;
	 for (int k = 0; k < _col; k++)
	    T[i].vec[j] += mat[i].vec[k] * M[k].vec[j];
      }

   for (i = 0; i < _row; i++)
      for (j = 0; j < _col; j++)
	 mat[i].vec[j] = T[i].vec[j];

   return *this;
}

// ޥȥꥯ黻(Ȥξ軻)
Mat& Mat::operator *= (const double a)
{
   for (int i = 0; i < _row; i++)
      for (int j = 0; j < _col; j++)
	 mat[i].vec[j] *= a;

   return *this;
}

// ޥȥꥯ黻(Ȥν)
Mat& Mat::operator /= (const double a)
{
   for (int i = 0; i < _row; i++)
      for (int j = 0; j < _col; j++)
	 mat[i].vec[j] /= a;

   return *this;
}

// ޥȥꥯȤν
Mat	operator / (const Mat& M, double a)
{
   Mat	T(M);

   for (int i = 0; i < M._row; i++)
      for (int j = 0; j < M._col; j++)
	 T.mat[i][j] /= a;

   return T;
}

// ޥȥꥯ֤ξ軻
Mat	operator * (const Mat& M1, const Mat& M2)
{
   Mat	T(M1._row, M2._col);

   assert(M1._col == M2._row);

   for (int i = 0; i < M1._row; i++)
      for (int j = 0; j < M2._col; j++)
      {
	 T.mat[i][j] = 0.0;
	 for (int k = 0; k < M1._col; k++)
	    T.mat[i][j] += M1.mat[i][k] * M2.mat[k][j];
      }
   return T;
}

// ޥȥꥯȤξ軻
Mat	operator * (const Mat& M, double a)
{
   Mat	T(M);

   for (int i = 0; i < T._row; i++)
      for (int j = 0; j < T._col; j++)
	 T.mat[i][j] *= a;

   return T;
}

// ޥȥꥯȤξ軻(2)
Mat	operator * (double a, const Mat& M)
{
   return (M * a);
}

// ޥȥꥯβû
Mat	operator + (const Mat& M1, const Mat& M2)
{
   Mat	T(M1);

   assert(M1._col == M2._col && M1._row == M2._row);
   for (int i = 0; i < M1._row; i++)
      for (int j = 0; j < M1._col; j++)
	 T.mat[i][j] += M2.mat[i][j];

   return T;
}

// ޥȥꥯθ
Mat	operator - (const Mat& M1, const Mat& M2)
{
   Mat	T(M1);

   assert(M1._col == M2._col && M1._row == M2._row);
   for (int i = 0; i < M1._row; i++)
      for (int j = 0; j < M1._col; j++)
	 T.mat[i][j] -= M2.mat[i][j];

   return T;
}

// ñΥޥʥ
Mat	operator - (const Mat& M)
{
   Mat	T(M);

   for (int i = 0; i < M._row; i++)
      for (int j = 0; j < M._col; j++)
	 T.mat[i][j] = -T.mat[i][j];

   return T;
}

// ޥȥꥯž
Mat	operator ~ (const Mat& M)
{
   Mat	T(M._col, M._row);

   for (int i = 0; i < T._row; i++)
      for (int j = 0; j < T._col; j++)
	 T.mat[i][j] = M.mat[j][i];

   return T;
}

// ޥȥꥯ
double	operator , (const Mat& M1, const Mat& M2)
{
   double	d = 0.0;

   assert(M1._col == M2._col && M1._row == M2._row);

   for (int i = 0; i < M1._row; i++)
      for (int j = 0; j < M1._col; j++)
	 d += M1.mat[i][j] * M2.mat[i][j];

   return d;
}

// ޥȥꥯ()
int	operator == (const Mat& M1, const Mat& M2)
{
   assert(M1.col() == M2.col() && M1.row() == M2.row());

   for (int i = 0; i < M1.row(); i++)
      for (int j = 0; j < M1.col(); j++)
	 if (fabs(M1[i][j] - M2[i][j]) > EPS)
	    return False;
   return True;
}

// ޥȥꥯ(԰)
int	operator != (const Mat& M1, const Mat& M2)
{
   assert(M1.col() == M2.col() && M1.row() == M2.row());

   for (int i = 0; i < M1.row(); i++)
      for (int j = 0; j < M1.col(); j++)
	 if (fabs(M1[i][j] - M2[i][j]) > EPS)
	    return True;
   return False;
}

// ޥȥꥯ٥ȥξ軻  ٥ȥ
VecC	operator * (const Mat& M, const VecC& V)
{
   VecC	T(M.row());

   assert(M.col() == V.size());

   for (int i = 0; i < M.row(); i++)
   {
      T[i] = 0.0;
      for (int j = 0; j < M.col(); j++)
	 T[i] += M[i][j] * V[j];
   }

   return T;
}

// ԥ٥ȥȥޥȥꥯξ軻  ԥ٥ȥ
VecR	operator * (const VecR& V, const Mat& M)
{
   VecR	T(M.col());

   assert(M.row() == V.size());

   for (int i = 0; i < M.col(); i++)
   {
      T[i] = 0.0;
      for (int j = 0; j < M.row(); j++)
	 T[i] += M[j][i] * V[j];
   }

   return T;
}

// ٥ȥ * ԥ٥ȥ => ޥȥꥯ
Mat	operator * (const VecC& CV, const VecR& RV)
{
   Mat	M(CV.size(), RV.size());

   for (int i = 0; i < CV.size(); i++)
      for (int j = 0; j < RV.size(); j++)
	 M[i][j] = CV[i] * RV[j];

   return M;
}

// ΥΥ
double	Mat::norm() const
{
   double	d = 0.0;

   for (int i = 0; i < _col; i++)
      for (int j = 0; j < _row; j++)
	 d += mat[i][j] * mat[i][j];

   return sqrt(d);
}


// Υ
double	Mat::sqnorm() const
{
   double	d = 0.0;

   for (int i = 0; i < _col; i++)
      for (int j = 0; j < _row; j++)
	 d += mat[i][j] * mat[i][j];

   return d;
}


// ϥڥ졼
istream& operator >> (istream& s, Mat& M)
{
   for (int i = 0; i < M._row; i++)
      s >> M[i];

   return s;
}

// ϥڥ졼
ostream& operator << (ostream& s, const Mat& M)
{
   s << "\n";
   for (int i = 0; i < M._row; i++)
      s << M[i] << "\n";
   
   return s;
}

// չ
Mat	inverse(const Mat& M)
{
   Mat		T(M);
   double	**a, **y;
   int		i, j, n;

   assert(M.col() == M.row());
   n = M.col();
   a = matrix(1, n, 1, n);	// recipes
   y = matrix(1, n, 1, n);	// recipes

   for (i = 0; i < n; i++)	// Ѥ˥ԡ
      for (j = 0; j < n; j++)
	 a[i+1][j+1] = M[i][j];

   rc_inverse(a, y, n);		// 饤֥(librecipes.a)θƤӽФ

   for (i = 0; i < n; i++)	// Ϥ饳ԡ
      for (j = 0; j < n; j++)
	 T[i][j] = y[i+1][j+1];

   free_matrix(a, 1, n, 1, n);	// recipes
   free_matrix(y, 1, n, 1, n);	// recipes

   return T;
}

// 33ι
double det33(const Mat& M)
{
   double	d;

   assert(M.row() == 3 && M.col() == 3);

   d = M[0][0] * M[1][1] * M[2][2]
     + M[0][2] * M[2][1] * M[1][0]
     + M[0][1] * M[1][2] * M[2][0]
     - M[0][2] * M[1][1] * M[2][0]
     - M[0][1] * M[1][0] * M[2][2]
     - M[0][0] * M[2][1] * M[1][2];

   return d;
}

// ޥȥꥯ(EPS꾮Τ0)
Mat ignore_EPS(const Mat& M0)
{
   Mat	M(M0.row(), M0.col());

   for (int i = 0; i < M0.row(); i++)
      for (int j = 0; j < M0.col(); j++)
	 M[i][j] = (fabs(M0[i][j]) < DET_EPS)? 0.0: M0[i][j];

   return M;
}

// 33ι(EPS꾮Τ0Ȥ)
double det33EPS(const Mat& M0)
{
   double	d;
   Mat		M;

   assert(M0.row() == 3 && M0.col() == 3);

   M = ignore_EPS(M0);

   d = M[0][0] * M[1][1] * M[2][2]
     + M[0][2] * M[2][1] * M[1][0]
     + M[0][1] * M[1][2] * M[2][0]
     - M[0][2] * M[1][1] * M[2][0]
     - M[0][1] * M[1][0] * M[2][2]
     - M[0][0] * M[2][1] * M[1][2];

   return d;
}

// ̤ι󼰤
double det(const Mat& M)
{
   double	**a, d;
   int		i, j, n;

   assert(M.row() == M.col());
   n = M.row();
   a = matrix(1, n, 1, n);	// recipes

   for (i = 0; i < n; i++)	// Ѥ˥ԡ
      for (j = 0; j < n; j++)
	 a[i+1][j+1] = M[i][j];

   d = rc_det(a, n);		// 饤֥(librecipes.a)θƤӽФ

   free_matrix(a, 1, n, 1, n);	// recipes
   return d;
}

// ͭͤȸͭ٥ȥ
// EV: ٥ȥ뤬ͭ٥ȥ
// V:  ͭ
void eigen_vector(const Mat& M, Mat& EV, VecC& V)
{
   double	**m, *d, *e;
   int		i, j, n;

   assert(M.row() == M.col());
   assert(EV.row() == EV.col() && EV.row() == M.row());
   assert(M.col() == V.size());

   n = M.row();
   m = matrix(1, n, 1, n);	// recipes
   d = vector(1, n);		// recipes
   e = vector(1, n);		// recipes

   for (i = 0; i < n; i++)	// Ѥ˥ԡ
      for (j = 0; j < n; j++)
	 m[i+1][j+1] = M[i][j];

   tred2(m, n, d, e);		// HouseholderѴ
   tqli(d, e, n, m);		// ŪեȤĤQLˡ
   eigsrt(d, m, n);		// ͭͤӸͭ٥ȥΥ

   for (i = 0; i < n; i++)	// Ϥ饳ԡ
   {
      V[i] = d[i+1];
      for (j = 0; j < n; j++)
	 EV[i][j] = m[i+1][j+1];
   }

   free_matrix(m, 1, n, 1, n);	// recipes
   free_vector(d, 1, n);	// recipes
   free_vector(e, 1, n);	// recipes
}

// ǾͭͤФͭ٥ȥ
VecC	min_eigen_vector(const Mat& M)
{
   Mat	EgVec(M);
   VecC	EgVal(M.col()), u(M.col());
   int	k;

   eigen_vector(M, EgVec, EgVal);

   k = M.col() - 1;
   for (int i = 0; i < EgVec.row(); i++)
      u[i] = EgVec[i][k];

   return u;
}

// ǾͭͤФͭ٥ȥȤθͭ
VecC	min_eigen_vector2(const Mat& M, double *l)
{
   Mat	EgVec(M);
   VecC	EgVal(M.col()), u(M.col());
   int	k;

   eigen_vector(M, EgVec, EgVal);

   k = M.col() - 1;
   for (int i = 0; i < EgVec.row(); i++)
      u[i] = EgVec[i][k];

   *l = EgVal[k];

   return u;
}

#if 0

// ðʬԤʤ
void SVD(const Mat& M, Mat& V, VecC& S, Mat& U)
{
   double	**v, *s, **u;
   int		i, j, n, m;

   assert(M.col() == V.col());
   assert(M.row() == V.row());
   assert(V.col() == S.size());
   assert(S.size() == U.col());  // U ž֤Ƴݤ뤫

   m = M.row();
   n = M.col();
   v = matrix(1, m, 1, n);	// recipes
   s = vector(1, n);		// recipes
   u = matrix(1, n, 1, n);	// recipes

   for (i = 0; i < m; i++)	// Ѥ˥ԡ
      for (j = 0; j < n; j++)
	 v[i+1][j+1] = M[i][j];

   svdcmp(v, m, n, s, u);	// ðʬ

   for (i = 0; i < m; i++)	// ѤϤ˥ԡ
      for (j = 0; j < n; j++)
	 V[i][j] = v[i+1][j+1];

   for (i = 0; i < n; i++)
   {
      S[i] = s[i+1];
      for (j = 0; j < n; j++)
         U[i][j] = u[i+1][j+1];
   }

   free_matrix(v, 1, m, 1, n);	// recipes
   free_vector(s, 1, n);	// recipes
   free_matrix(u, 1, n, 1, n);	// recipes
}

#endif

// LU ʬˤϢΩ켡β
VecC    SLE(const Mat& A, const VecC& B)
{
   VecC         X(B.size());
   double       **a, *b, d;
   int          i, j, n, *indx;

   assert(A.col() == A.row());
   assert(A.col() == B.size());

   n = A.col();

   a = matrix(1, n, 1, n);
   b = vector(1, n);
   indx = ivector(1, n);

   for (i = 0; i < n; i++)
   {
      b[i+1] = B[i];
      for (j = 0; j < n; j++)
         a[i+1][j+1] = A[i][j];
   }

   ludcmp(a, n, indx, &d);
   lubksb(a, n, indx, b);

   for (i = 0; i < n; i++)
      X[i] = b[i+1];

   free_matrix(a, 1, n, 1, n);
   free_vector(b, 1, n);
   free_ivector(indx, 1, n);

   return X;
}

// ̵չ
Mat	gen_inv(const Mat& M)
{
   Mat	T(M.col(), M.row());
   double	**a, **x;
   int		i, j, m, n;

   m = M.row();
   n = M.col();
   a = matrix(1, m, 1, n);	// recipes
   x = matrix(1, n, 1, m);	// recipes

   for (i = 0; i < m; i++)	// Ѥ˥ԡ
      for (j = 0; j < n; j++)
	 a[i+1][j+1] = M[i][j];

   rc_gen_inv(a, x, m, n);	// 饤֥(librecipes.a)

   for (i = 0; i < n; i++)	// Ϥ饳ԡ
      for (j = 0; j < m; j++)
	 T[i][j] = x[i+1][j+1];

   free_matrix(a, 1, m, 1, n);	// recipes
   free_matrix(x, 1, n, 1, m);	// recipes

   return T;
}
// ݤ̵չ
Mat	trc_gen_inv(const Mat& M, int r)
{
   Mat	T(M.col(), M.row());
   double	**a, **x;
   int		i, j, m, n;

   m = M.row();
   n = M.col();
   a = matrix(1, m, 1, n);	// recipes
   x = matrix(1, n, 1, m);	// recipes

   for (i = 0; i < m; i++)	// Ѥ˥ԡ
      for (j = 0; j < n; j++)
	 a[i+1][j+1] = M[i][j];

   rc_trunc_gen_inv(a, x, m, n, r);	// 饤֥(librecipes.a)

   for (i = 0; i < n; i++)	// Ϥ饳ԡ
      for (j = 0; j < m; j++)
	 T[i][j] = x[i+1][j+1];

   free_matrix(a, 1, m, 1, n);	// recipes
   free_matrix(x, 1, n, 1, m);	// recipes

   return T;
}

// nnñ̹
Mat
id_matrix(int n = 3)
{
   Mat	T(n, n);
 
   T.clear(0);
   for (int i = 0; i < n; i++)
      T[i][i] = 1.0;

   return T;
}

// оβ
Mat
sym(const Mat& M)
{
   return (M + (~M)) / 2.0;
}

// 3٥ȥ33Ȥγ
Mat
operator % (const VecC& v, const Mat& M)
{
   VecC	x0, x1, x2;
   Mat	T(3, 3);

   assert(M.col() == M.row() && M.col() == 3);
   assert(v.size() == 3);

   x0 = v % M(0);
   x1 = v % M(1);
   x2 = v % M(2);
   T.setcvec(0, x0);
   T.setcvec(1, x1);
   T.setcvec(2, x2);

   return T;
}

// 333٥ȥȤγ
Mat
operator % (const Mat& M, const VecC& v)
{
   Mat	I = id_matrix(3);
   return (M * (~(v % I)));
}

// 33γ(exterior product)
Mat
ext_product(const Mat& A, const Mat& B)
{
   Mat	T(3,3);

   assert(A.col() == A.row() && A.col() == 3);
   assert(B.col() == B.row() && B.col() == 3);

   T[0][0] =   A[1][1] * B[2][2]- A[1][2] * B[2][1]- A[2][1] * B[1][2]+ A[2][2] * B[1][1];
   T[0][1] = - A[1][0] * B[2][2]+ A[1][2] * B[2][0]+ A[2][0] * B[1][2]- A[2][2] * B[1][0];
   T[0][2] =   A[1][0] * B[2][1]- A[1][1] * B[2][0]- A[2][0] * B[1][1]+ A[2][1] * B[1][0];
   T[1][0] = - A[0][1] * B[2][2]+ A[0][2] * B[2][1]+ A[2][1] * B[0][2]- A[2][2] * B[0][1];
   T[1][1] =   A[0][0] * B[2][2]- A[0][2] * B[2][0]- A[2][0] * B[0][2]+ A[2][2] * B[0][0];
   T[1][2] = - A[0][0] * B[2][1]+ A[0][1] * B[2][0]+ A[2][0] * B[0][1]- A[2][1] * B[0][0];
   T[2][0] =   A[0][1] * B[1][2]- A[0][2] * B[1][1]- A[1][1] * B[0][2]+ A[1][2] * B[0][1];
   T[2][1] = - A[0][0] * B[1][2]+ A[0][2] * B[1][0]+ A[1][0] * B[0][2]- A[1][2] * B[0][0];
   T[2][2] =   A[0][0] * B[1][1]- A[0][1] * B[1][0]- A[1][0] * B[0][1]+ A[1][1] * B[0][0];

   return T;
}

// 33;ҹ
Mat cofactor33(const Mat& M)
{
   Mat	T(3,3);

   T[0][0] = M[1][1] * M[2][2] - M[2][1] * M[1][2];
   T[0][1] = M[2][1] * M[0][2] - M[0][1] * M[2][2];
   T[0][2] = M[0][1] * M[1][2] - M[1][1] * M[0][2];
   T[1][0] = M[1][2] * M[2][0] - M[2][2] * M[1][0];
   T[1][1] = M[2][2] * M[0][0] - M[0][2] * M[2][0];
   T[1][2] = M[0][2] * M[1][0] - M[1][2] * M[0][0];
   T[2][0] = M[1][0] * M[2][1] - M[2][0] * M[1][1];
   T[2][1] = M[2][0] * M[0][1] - M[0][0] * M[2][1];
   T[2][2] = M[0][0] * M[1][1] - M[1][0] * M[0][1];

   return T;
}

// 33ޥȥꥯ3Ĥ٥ȥʬ
void col3(const Mat& M, VecC& u1, VecC& u2, VecC& u3)
{
   int	i;

   for (i = 0; i < 3; i++)
   {
      u1[i] = M[i][0];
      u2[i] = M[i][1];
      u3[i] = M[i][2];
   }
}

// (ե٥˥Υ1ˤ)
Mat normalize(const Mat& M)
{
   Mat		T(M);
   double	d;

   if ((d = T.norm()) < EPS)
   {
      T.clear(0.0);
      return T;
   }
   return T / d;
}

// Υȥ졼
double trace(const Mat& M)
{
   double	t;
   
   assert(M.col() == M.row());

   t = 0.0;
   for (int i = 0; i < M.row(); i++)
      t += M[i][i];

   return t;
}

// Ϳ줿ͤгǤȤ33
Mat diag33(double a, double b, double c)
{
   Mat  T;

   T[0][0] = a;
   T[1][1] = b;
   T[2][2] = c;

   return T;
}

// ٥ȥǤгǤȤ
Mat diag(const Vec& V)
{
   Mat  T;

   for(int i = 0; i < V.size(); i++)
      T[i][i] = V[i];

   return T;
}

// Ϳ줿гǤΤߤι
Mat diag(const Mat& M)
{
   Mat  T(M.row(), M.col());

   for(int i = 0; i < M.row(); i++)
      for(int j = 0; j < M.col(); j++)
	 T[i][i] = M[i][i];

   return T;
}
