//
// ѴκŬ
//

#include <iostream>
#include <complex>
#include <cmath>

#include        "Mat.h"
#include        "Tensor.h"
#include        "Cast.h"

#include        "similar.h"

using namespace std;

typedef	complex<double>	Complex;

// 襤饤ؿ
static inline double
sqr(double x)
{
   return x * x;
}

// ˥塼ȥˡνͤη׻
static int
init_similar_sub(VecC* x0, VecC* x1, int N,
                 double *s, double *theta, double *t1, double *t2)
{
   // ſη׻
   Complex      zC0(x0[0][0], x0[0][1]), zC1(x1[0][0], x1[0][1]);

   for (int i = 1; i < N; i++)
   {
      Complex   z0(x0[i][0], x0[i][1]), z1(x1[i][0], x1[i][1]);
 
      zC0 += z0;
      zC1 += z1;
   }
   zC0 /= (double)N;
   zC1 /= (double)N;

   // ſѰ
   Complex      tt(0.0, 0.0), zz;
   double       ts = 0.0;

   for (int i = 0; i < N; i++)
   {
      Complex   z0(x0[i][0], x0[i][1]), z1(x1[i][0], x1[i][1]);
 
      zz = (z1 - zC1) / (z0 - zC0);
      ts += abs(zz);
      tt += zz / abs(zz);
   }

   *s = ts / (double)N;
   *theta = arg(tt / (double)N);

   Complex tau = zC1 - polar(*s, *theta) * zC0;
   *t1 = real(tau);
   *t2 = imag(tau);

   return 0;
}

int
init_similar(VecC* x0, VecC* x1, int N, Mat& S)
{
   double       s, theta, t1, t2;

   init_similar_sub(x0, x1, N, &s, &theta, &t1, &t2);

   S[0][0] = S[1][1] = s * cos(theta);
   S[0][1] = -(S[1][0] = s * sin(theta));
   S[0][2] =  t1;
   S[1][2] =  t2;
   S[2][0] = S[2][1] = 0.0;
   S[2][2] = 1.0;

   return 0;
}

static Mat
make_S(double s, double theta, VecC& T)
{
   Mat  S;

   S[0][0] = S[1][1] = s * cos(theta);
   S[0][1] = -(S[1][0] = s * sin(theta));
   S[0][2] =  T[0];
   S[1][2] =  T[1];
   S[2][0] = S[2][1] = 0.0;
   S[2][2] = 1.0;

   return S;
}

static Mat
make_R(double theta)
{
   Mat  R(2,2);

   R[0][0] = R[1][1] = cos(theta);
   R[0][1] = -(R[1][0] = sin(theta));

   return R;
}

// Ť߹צη׻
static Mat
calc_Wa(Mat& S)
{
   Mat  Pk(1.0, 0.0, 0.0,
	   0.0, 1.0, 0.0,
	   0.0, 0.0, 0.0);
   Mat	T, Wa;

   T = S * Pk * (~S) + Pk;
   Wa = trc_gen_inv(T, 2);

   return Wa;
}

// 33κ22ʬȴФ
static Mat
submatrix22(Mat& H)
{
   Mat  T(2,2);

   if (H.row() != 3 || H.col() != 3)
   {
      cerr << "submatrix: wrong matrix size!" << endl;
      exit(1);
   }

   T[0][0] = H[0][0];
   T[0][1] = H[0][1];
   T[1][0] = H[1][0];
   T[1][1] = H[1][1];

   return T;
}

// 3٥ȥ2ʬȴФ
static VecC
subvector2(VecC& V)
{
   VecC T(2);

   if (V.size() != 3)
   {
      cerr << "subvector: wrong vector size!" << endl;
      exit(1);
   }

   T[0] = V[0];
   T[1] = V[1];

   return T;
}

// ۢʤη׻
static VecC
grad_J(VecC* x0, VecC* x1, Mat Wa, int N,
       Mat& R, double s, VecC& T)
{
   Mat          J(2,2), wa(2,2);
   VecC         x0p(2), x1p(2), ea(2), nJt(2), nJ(4), t(2);
   double       dJs, dJt;

   J[0][0] = J[1][1] = 0.0;
   J[0][1] = - (J[1][0] = 1.0);
   wa = submatrix22(Wa);

   nJt.clear(0.0);
   dJs = dJt = 0.0;
   for (int i = 0; i < N; i++)
   {
      x0p = subvector2(x0[i]);
      x1p = subvector2(x1[i]);
      ea = x1p - s * R * x0p - T;

      dJs += ((R * x0p) , (wa * ea));
      dJt += ((J * R * x0p) , (wa * ea));
      nJt += wa * ea;
   }
   dJs *=  -2.0 / (double)N;
   dJt *=  -2.0 * s / (double)N;
   nJt *=  -2.0 / (double)N;

   nJ[0] = dJs;
   nJ[1] = dJt;
   nJ[2] = nJt[0];
   nJ[3] = nJt[1];

//   cerr << " = " << nJ << endl;

   return nJ;
}

// إåη׻
static Mat
hessian_S(VecC *x0, Mat Wa, int N,
           Mat& R, double s)
{
   Mat          na2J(4,4);
   double       J2s2, J2st, J2t2;
   VecC         nJs(2), nJt(2);
   Mat          n2Jt(2,2);
   Mat          J(2,2);
   VecC         xa(2), Rx(2), JRx(2);
   Mat          wa(2,2);

   J2s2 = J2st = J2t2 = 0.0;
   nJs.clear(0.0);
   nJt.clear(0.0);
   n2Jt.clear(0.0);
   J[0][0] = J[1][1] = 0.0;
   J[0][1] = -(J[1][0] = 1.0);

   wa = submatrix22(Wa);

   for (int i = 0; i < N; i++)
   {
      xa = subvector2(x0[i]);
      Rx = R * xa;
      JRx = J * Rx;

      J2s2 += (Rx , (wa * Rx));
      J2t2 += (JRx , (wa * JRx));
      J2st += (Rx , (wa * JRx));
      nJs += wa * Rx;
      nJt += wa * JRx;
      n2Jt += wa;
   }
   J2s2 *= 2.0 / (double)N;
   J2st *= 2.0 * s / (double)N;
   J2t2 *= 2.0 * s * s / (double)N;
   nJs  *= 2.0 / (double)N;
   nJt  *= 2.0 * s / (double)N;
   n2Jt *= 2.0 / (double)N;

   na2J[0][0] = J2s2;
   na2J[1][1] = J2t2;
   na2J[0][1] = na2J[1][0] = J2st;
   na2J[0][2] = na2J[2][0] = nJs[0];
   na2J[0][3] = na2J[3][0] = nJs[1];
   na2J[1][2] = na2J[2][1] = nJt[0];
   na2J[1][3] = na2J[3][1] = nJt[1];
   na2J[2][2] = n2Jt[0][0];
   na2J[2][3] = n2Jt[0][1];
   na2J[3][2] = n2Jt[1][0];
   na2J[3][3] = n2Jt[1][1];

   return na2J;
}

//
// ѴΥ˥塼ȥˡˤ׻
//
double
calc_similar(VecC* x0, VecC* x1, int N,
	     double eps_s, double eps_th, double eps_T, int max_itr,
	     double *scale, Mat& Rot, VecC& tr,
	     double *eh, int *count)
{
   double       J;
   VecC         naJ(4);
   Mat          na2J(4,4);
   VecC         dS(4);
   Mat          R(2,2);
   VecC         T(2), dT(2);
   Mat          S;
   Mat		Wa;
   double       s, theta, t1, t2;
   double       ds, dtheta, dTn;
   VecC         xSx;
   Mat          EgVec(4,4);
   VecC         EgVal(4);
   int          iter;

   // ͤη׻
   init_similar_sub(x0, x1, N, &s, &theta, &t1, &t2);
   T[0] = t1; T[1] = t2;
   R = make_R(theta);
   S = make_S(s, theta, T);

   Wa = calc_Wa(S);
   J = 0.0;
   for (int i = 0; i < N; i++)
   {
      xSx = x1[i] -  S * x0[i];
      J += (xSx , (Wa * xSx));
   }
   J /= (double)N;

   iter = 0;
   do
   {
      iter++;

      if (iter > max_itr)
      {
         cerr << "similarity fitting: iteration over! " << endl;
         *count = max_itr;
         return ERR_J;
      }

      // ۢʤη׻
      naJ = grad_J(x0, x1, Wa, N, R, s, T);

      // إå2ʤη׻
      na2J = hessian_S(x0, Wa, N, R, s);

      eigen_vector(na2J, EgVec, EgVal);

      // ϢΩ켡
      dS = - inverse(na2J) * naJ;

      ds = dS[0];
      dtheta = dS[1];
      dT[0] = dS[2];
      dT[1] = dS[3];

      s += ds;
      theta += dtheta;
      T += dT;
      R = make_R(theta);
      S = make_S(s, theta, T);

      dTn = dT.norm();

      // צη׻
      Wa = calc_Wa(S);

      J = 0.0;
      for (int i = 0; i < N; i++)
      {
         xSx = x1[i] -  S * x0[i];
         J += (xSx , (Wa * xSx));
      }
      J /= (double)N;

   }
   while (fabs(ds) >= eps_s || fabs(dtheta) >= eps_th || fabs(dTn) >= eps_T);

   J = 0.0;
   for (int i = 0; i < N; i++)
   {
      xSx = x1[i] -  S * x0[i];
      J += (xSx , (Wa * xSx));
   }
   J /= (double)N;

   *eh = J / (2.0 - 4.0 / N);
   *count = iter;

   *scale = s;
   Rot = R;
   tr = T;

   return J;
}

//
// Ѵ Levenberg-Marquardt ˡˤ׻
//
double
calc_similarlm(VecC* x0, VecC* x1, int N,
	       double eps_s, double eps_th, double eps_T, int max_itr,
	       double *scale, Mat& Rot, VecC& tr,
	       double *eh, int *count)
{
   double       J;
   Mat          Wa;
   VecC         naJ(4);
   Mat          na2J(4,4);
   VecC         dS(4);
   Mat          R(2,2);
   VecC         T(2), dT(2);
   Mat          S;
   double       s, theta, t1, t2;
   double       ds, dtheta, dTn;
   VecC         xSx;
   Mat          EgVec(4,4);
   VecC         EgVal(4);
   int          iter;
   // LM ˡΤѿ
   Mat		na2J1(4,4), DD(4,4);
   Mat		SS, RR(2,2);
   VecC		TT(2);
   double	lambda, J1, J2, dJ, ss, ttheta;
   int		lm_flag;

   // ͤη׻
   init_similar_sub(x0, x1, N, &s, &theta, &t1, &t2);
   T[0] = t1; T[1] = t2;
   R = make_R(theta);
   S = make_S(s, theta, T);

   Wa = calc_Wa(S);
   J = 0.0;
   for (int i = 0; i < N; i++)
   {
      xSx = x1[i] -  S * x0[i];
      J += (xSx , (Wa * xSx));
   }
   J /= (double)N;
   
   // LM ˡΤν
   J1 = J;
   lambda = Init_Lambda;
   
   iter = 0;
   do
   {
      iter++;
      lm_flag = 0;

      if (iter > max_itr)
      {
         cerr << "LM similarity fitting: iteration over! " << endl;
         *count = max_itr;
         return ERR_J;
      }

      // ۢʤη׻
      naJ = grad_J(x0, x1, Wa, N, R, s, T);

      // إå2ʤη׻
      na2J = hessian_S(x0, Wa , N, R, s);
      DD = diag(na2J);

      // LM Loop
      do
      {
	 if (lm_flag)
	    lambda *= Mult_Lambda;
	 else
	    lm_flag++;

	 na2J1 = na2J + lambda * DD;
      
	 // ϢΩ켡
	 dS = SLE(na2J1, -naJ);

	 ds = dS[0];
	 dtheta = dS[1];
	 dT[0] = dS[2];
	 dT[1] = dS[3];

	 ss = s + ds;
	 ttheta = theta + dtheta;
	 TT = T + dT;
	 RR = make_R(ttheta);
	 SS = make_S(ss, ttheta, TT);

	 // צη׻
	 Wa = calc_Wa(SS);

	 J2 = 0.0;
	 for (int i = 0; i < N; i++)
	 {
	    xSx = x1[i] -  SS * x0[i];
	    J2 += (xSx , (Wa * xSx));
	 }
	 J2 /= (double)N;
      }
      while (J2 >= J1);

      dJ = J1 - J2;
      J1 = J2;
      S = SS;
      R = RR;
      T = TT;
      s = ss;
      theta = ttheta;

      dTn = dT.norm();
   }
   while (fabs(ds) >= eps_s || fabs(dtheta) >= eps_th || fabs(dTn) >= eps_T);

   J = 0.0;
   for (int i = 0; i < N; i++)
   {
      xSx = x1[i] -  S * x0[i];
      J += (xSx , (Wa * xSx));
   }
   J /= (double)N;

   *eh = J / (2.0 - 4.0 / N);
   *count = iter;

   *scale = s;
   Rot = R;
   tr = T;

   return J;
}
